%pure_parser
%{
// Rascal, the Advanced Scientific CALculator
// Copyright (C) 2001, Sebastian Ritterbusch (Rascal@Ritterbusch.de)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more detauls.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "symtab.h"
#include "value.hpp"
%}
%union {
	value  *val;
	symrec *tptr;
       }
%{
/* 

*/
#include <math.h>
#include <stdlib.h>
#include <string.h>
int precis=8;

void * set_string_input(char *);
void   reset_input(void *);
value	result;
void yyerror(char *s);
int yylex(YYSTYPE *yylval);
%}
%token <val> PARSERnumber 
%token <tptr> PARSERvariable PARSERfunction PARSERfunction2 PARSERuserfunction PARSERprocedure
%token PARSERfunctionDefinition
%token PARSERbecomes PARSERlparent PARSERrparent PARSERnewline
%token PARSERlbracket PARSERrbracket PARSERcomma
%token PARSERat
%left PARSERwhite
%left PARSERbecomes
%left PARSERsemicolon
%left PARSERML
%left PARSERquestionmark
%left PARSERcolon
%left PARSERLT PARSERLE PARSERGT PARSERGE
%left PARSEREQ PARSERNEQ
%left PARSERAND
%left PARSEROR
%left PARSERplus PARSERminus
%left PARSERtimes PARSERdivide PARSERmodulo
%right PARSERpower PARSERtranspose PARSERFAC PARSERNOT
%right PARSERlparent
%type <val> PARSERexpr
%type <val> PARSERmatrix
%type <val> PARSERmatrixline
%%
PARSERlist:	{ }
	| PARSERlist PARSERexpr PARSERnewline
			{ 
                           result=*($2);
			   value o(output(*($2)));
			   if(o.isSTRING())
			      printf(" %s\n",o.asSTRING().c_str());
			   else
			      printf(" Error.\n");
			}
	| PARSERlist PARSERexpr PARSERsemicolon
			{ result=*($2); }
	| PARSERlist PARSERfunctionDefinition
			{ }
	| PARSERlist PARSERuserfunction PARSERnewline
			{
			   printf("%s%s\n",$2->name,$2->ivalue.body); 
			}
        | PARSERlist PARSERfunction PARSERnewline
                        {
                           printf("%s is a built-in unary function.\n",$2->name);
                        }
        | PARSERlist PARSERfunction2 PARSERnewline
                        {
                           printf("%s is a built-in binary function.\n",$2->name);
                        }
	| PARSERlist PARSERnewline 
			{ printf("");}
	| PARSERlist error PARSERnewline
			{   yyerrok;
		            yyclearin;
			}
	;
PARSERexpr:
	  PARSERexpr PARSERwhite { $$ = $1; }	
	| PARSERfunction	{ $$ = new value($1) }
	| PARSERfunction2	{ $$ = new value($1) }
	| PARSERat PARSERprocedure	{ $$ = new value($2) }
	| PARSERuserfunction	{ $$ = new value($1) }
	| PARSERat PARSERvariable	{ $$ = new value($2) }
	| PARSERlbracket PARSERexpr PARSERcomma PARSERexpr PARSERrbracket { $$=new value(bracket(*($2),*($4))); delete $2; delete $4; }
        | PARSERwhite PARSERexpr %prec PARSERbecomes	{ $$ = $2; }
	| PARSERnumber			{ $$ = $1; }
	| PARSERvariable		{ $$ = new value($1->var); }
        | PARSERvariable PARSERbecomes PARSERexpr { $$ = $3; $1->var=*($3);}
	| PARSERprocedure 		{ $$=new value( (*($1->ivalue.procptr))() ); }
	| PARSERexpr PARSERlparent PARSERexpr PARSERrparent	
        {
	   if($1->isSYMREC())
	      $$=new value(evaluate($1->asSYMREC(),*$3)); 
	   else 
	      $$=new value( cell(*$1,*$3)); 
	   delete $1;delete $3; 
	}
	| PARSERexpr PARSERlbracket PARSERmatrix PARSERrbracket	
        {
	   if($1->isSYMREC())
	      $$=new value(evaluate($1->asSYMREC(),*$3)); 
	   else 
	      $$=new value( cell(*$1,*$3)); 
	   delete $1;delete $3; 
	}
	| PARSERexpr PARSERlparent PARSERexpr PARSERcomma PARSERexpr PARSERrparent	
        {
	   if($1->isSYMREC())
	      $$=new value(evaluate($1->asSYMREC(),*$3,*$5)); 
	   else 
	      $$=new value( cell(cell(*$1,*$3),*$5)); 
	   delete $1;delete $3; delete $5;
	}
	| PARSERexpr PARSERplus PARSERexpr	{ $$ = new value(add(*($1),*($3))); delete $1;delete $3; }
	| PARSERexpr PARSERminus PARSERexpr	{ $$ = new value(sub(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSERtimes PARSERexpr	{ $$ = new value(mul(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSERdivide PARSERexpr	{ $$ = new value(div(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSERmodulo PARSERexpr	{ $$ = new value(mod(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSERpower PARSERexpr	{ $$ = new value(pow(*($1),*($3)));delete $1;delete $3;}
	| PARSERexpr PARSERLT PARSERexpr	{ $$ = new value(less(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSERLE PARSERexpr	{ $$ = new value(lesseq(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSERGT PARSERexpr	{ $$ = new value(greater(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSERGE PARSERexpr	{ $$ = new value(greatereq(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSEREQ PARSERexpr	{ $$ = new value(eq(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSERNEQ PARSERexpr	{ $$ = new value(neq(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSERAND PARSERexpr	{ $$ = new value(And(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSEROR PARSERexpr	{ $$ = new value(Or(*($1),*($3))); delete $1;delete $3;}
	| PARSERexpr PARSERquestionmark PARSERexpr PARSERcolon PARSERexpr
				{ $$ = new value((($1)->isINTEGER() && ($1)->asINTEGER())?*($3):*($5)); delete $1;delete $3;delete $5;}
	| PARSERNOT PARSERexpr		{ $$ = new value(Not(*($2)));delete $2; }
	| PARSERexpr PARSERFAC		{ $$ = new value(fac(*($1))); delete $1;}
	| PARSERexpr PARSERtranspose	{ $$ = new value(transpose(*($1))); delete $1; }
	| PARSERlparent PARSERexpr PARSERrparent	{ $$ = $2; }
	| PARSERminus PARSERexpr %prec PARSERtimes { $$ = new value(neg(*($2)));delete $2;}
	| PARSERlbracket PARSERmatrix PARSERrbracket { $$=$2; }
	;
PARSERmatrix:	PARSERmatrixline		{ $$ = $1; }
	| PARSERmatrix PARSERsemicolon PARSERmatrixline { $$ = new value(appendline($1->asMATRIX(),$3->asMATRIX() )); delete $1; delete $3;}
	| PARSERmatrix PARSERsemicolon PARSERexpr	{ $$ = new value(appendline($1->asMATRIX(),valuematrix(*$3) )); delete $1; delete $3; }
	;
PARSERmatrixline: PARSERmatrixline PARSERwhite PARSERexpr	%prec PARSERML { $$ = new value(appendcol($1->asMATRIX(),valuematrix(*($3)))); delete $1; delete $3;}
	| PARSERexpr                  %prec PARSERML { $$=new value(valuematrix(*$1)); delete $1; }
	;
%%

#include <readline/readline.h>
#include <readline/history.h>

extern char *xmalloc(int);

char * dupstr(char *s)
{
   char *r=(char *)malloc(strlen(s)+1);
   strcpy(r,s);
   return r;
}

char * dupstr_function(char *s)
{
   char *r=(char *)malloc(strlen(s)+2);
   strcpy(r,s);
   strcat(r,"(");
   return r;
}

char * command_generator(char *text,int state)
{
   static symrec * current;
   static int      len;
   if(state==0)
   {
      current=sym_table;
      len=strlen(text);
   }
   while(current)
   {
      if(len==0 || strncmp(current->name,text,len)==0)
      {
         char *p=current->name;
         int  type=current->type;
         current=current->next;
         if(type==PARSERvariable || type==PARSERprocedure)
            return dupstr(p);
         else
            return dupstr_function(p);
      }
      current=current->next;
   }
   return ((char *)NULL);
}

char **rascal_completition(char *text,int start,int end)
{
   return completion_matches(text,(CPFunction *)command_generator);
}

void initialize_readline(void)
{
   rl_readline_name="Rascal";
   rl_attempted_completion_function=(CPPFunction *)rascal_completition;
   rl_basic_word_break_characters="+-*/; ^|&<>=?:(),";
   init_table();
}

main()
{
   // Without readline just:  yyparse();
   // but with... 

   cout << "Rascal " << VERSION;
   cout << " (C) 2001 Sebastian Ritterbusch" << endl;
   cout << endl;
   cout << "Enter 'help' for help or 'licence' for information about licence and warranty." << endl;
   cout << endl;

   initialize_readline();

   while(cin)
   {
      char *input=0;
      do {
         if(input)
            free(input);
         input=readline(">");
         if(!input)
            return 0;
      } while(!input || input[0]==0);
      
      string input_lf(input+string("\n"));
      
      void *buffer=set_string_input((char *)(input_lf.c_str()));
      yyparse();
      value res(output(result));
      reset_input(buffer);
      
      add_history(input);
      if(res.isSTRING() && res.asSTRING()!="Error")
         add_history((char *)(res.asSTRING().c_str()));      
   }
}

void yyerror(char *s)
 {
   printf("%s\n",s);
 }
